//
//  IHRGBADataView.m
//
//  Created by iOS Hero on 12/23/11.
//  Copyright (c) 2011 iOS Hero. All rights reserved.
//

#import "IHRGBADataView.h"

#define kShowSample YES
#define kShowTracks YES

@interface IHRGBADataView ()
{ 
    UIView * sampleView;    
    RGBAPixel * pixelData;
}

- (void)cachePixelData;
- (void)showTrack:(CGPoint)point;

@end

@implementation IHRGBADataView

- (void)awakeFromNib
{
    if(kShowSample)
    {
        //draw a color sample on the top left
        sampleView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, 40, 40)];            
        [self addSubview:sampleView];        
    }
    
    [self cachePixelData];
}

- (UIColor *)colorAtPoint:(CGPoint)point
{
    if(kShowTracks)
    {
        [self showTrack:point];
    }
    
    RGBAPixel pixel = {0, 0, 0, 0}; //make sure you initialize the pixel data
    
    CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceRGB();
    
    //create a 1x1 context where we will draw the image pixel
    CGContextRef context = CGBitmapContextCreate(&pixel, 1, 1, 8, sizeof(RGBAPixel), colorSpace,
                                                 //http://en.wikipedia.org/wiki/Endianness                                                 
                                                 kCGImageAlphaPremultipliedLast
                                                 | kCGBitmapByteOrder32Big);
    
    //make it the current context for next drawing operations
    UIGraphicsPushContext(context);
    
    [imageView.image drawAtPoint:CGPointMake(-point.x, -point.y)];
    
    //cleanup
    UIGraphicsPopContext();
    CGContextRelease(context);
    CGColorSpaceRelease(colorSpace);
    
    CGFloat red = pixel.red / 255.0f;
    CGFloat green = pixel.green / 255.0f;
    CGFloat blue = pixel.blue / 255.0f;
    CGFloat alpha = pixel.alpha / 255.0f;            
    
    //remember that color is premultiplied, get the rgb values separately to render colors with alpha properly
    UIColor * color = [UIColor colorWithRed:red / alpha green:green / alpha blue:blue / alpha alpha:alpha];
    
    if(kShowSample)
    {
        sampleView.backgroundColor = color;
    }
    
    return color;
}

- (UIColor *)colorAtPointUsingCache:(CGPoint)point
{
    if(! pixelData)
    {
        NSLog(@"pixel data cache has not been initialized");
        return [UIColor colorWithWhite:0 alpha:0];
    }
    
    if(kShowTracks)
    {
        [self showTrack:point];
    }
    
    CGImageRef imageRef = imageView.image.CGImage;
    float width = CGImageGetWidth(imageRef);
    float height = CGImageGetHeight(imageRef);
    
    //retina, non-retina into account
    float scale = [[UIScreen mainScreen] scale];
    float x = point.x * scale;
    float y = point.y * scale;
    
    UIColor * color;
    
    //watch the bounds otherwise it might wrap around
    if(x >= width || y >= height)
    {
        //return clear color
        color = [UIColor colorWithWhite:0 alpha:0];
    }
    else
    {
        int byteIndex = y * width + x;
        
        RGBAPixel pixel = pixelData[byteIndex];        
        CGFloat red = pixel.red / 255.0f;
        CGFloat green = pixel.green / 255.0f;
        CGFloat blue = pixel.blue / 255.0f;
        CGFloat alpha = pixel.alpha / 255.0f;            
        
        //remember that color is premultiplied, get the rgb values separately to render colors with alpha properly
        color = [UIColor colorWithRed:red / alpha green:green / alpha blue:blue / alpha alpha:alpha];
    }
    
    if(kShowSample)
    {
        sampleView.backgroundColor = color;
    }
    
    return color;    
}

- (void)cachePixelData
{
    CGImageRef imageRef = imageView.image.CGImage;
    int width = CGImageGetWidth(imageRef);
    int height = CGImageGetHeight(imageRef);
    int bitsPerComponent = 8;
    
    int byteCount = width * height * sizeof(RGBAPixel);
    pixelData = malloc(byteCount);
    
    CGBitmapInfo bitmapInfo = kCGImageAlphaPremultipliedLast | kCGBitmapByteOrder32Big;
    
    int bytesPerRow = width * sizeof(RGBAPixel);
    CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceRGB();
    CGContextRef context = CGBitmapContextCreate(pixelData, width, height, bitsPerComponent, bytesPerRow, colorSpace, bitmapInfo);
    
    //makes sure that the context is a copy of the image
    CGContextSetBlendMode(context, kCGBlendModeCopy);
    CGContextDrawImage(context, CGRectMake(0, 0, width, height), imageRef);
    
    CGContextRelease(context);
    CGColorSpaceRelease(colorSpace);
}

- (void)showTrack:(CGPoint)point
{
    UIView * track = [[UIView alloc] initWithFrame:CGRectMake(point.x, point.y, 1, 1)];
    track.backgroundColor = [UIColor colorWithWhite:0 alpha:0.5];
    [self addSubview:track];
    [track performSelector:@selector(removeFromSuperview) withObject:nil afterDelay:2];
}

- (void)dealloc
{
    free(pixelData);
}

@end
